"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const { __ΩPrimaryKey } = require('@deepkit/type');
const { __ΩAutoIncrement } = require('@deepkit/type');
const { __ΩReference } = require('@deepkit/type');
const { __ΩEntity } = require('@deepkit/type');
const globals_1 = require("@jest/globals");
const sqlite_platform_1 = require("../src/sqlite-platform");
const factory_1 = require("./factory");
const orm_integration_1 = require("@deepkit/orm-integration");
const sqlite_adapter_1 = require("../src/sqlite-adapter");
const core_1 = require("@deepkit/core");
const type_1 = require("@deepkit/type");
const orm_1 = require("@deepkit/orm");
(0, globals_1.test)('reflection circular reference', () => {
    const user = type_1.ReflectionClass.from(orm_integration_1.User);
    const credentials = type_1.ReflectionClass.from(orm_integration_1.UserCredentials);
    (0, globals_1.expect)(user.getProperty('credentials').isBackReference()).toBe(true);
    (0, globals_1.expect)(user.getProperty('credentials').getResolvedReflectionClass()).toBe(credentials);
    const userProperty = credentials.getProperty('user');
    (0, globals_1.expect)(userProperty.getResolvedReflectionClass()).toBe(user);
    (0, globals_1.expect)(userProperty.isPrimaryKey()).toBe(true);
    (0, globals_1.expect)(userProperty.isReference()).toBe(true);
});
(0, globals_1.test)('tables', () => {
    const [user] = new sqlite_platform_1.SQLitePlatform().createTables(orm_1.DatabaseEntityRegistry.from([orm_integration_1.User]));
    (0, globals_1.expect)(user.getColumn('birthdate').isNotNull).toBe(false);
    const [userCredentials] = new sqlite_platform_1.SQLitePlatform().createTables(orm_1.DatabaseEntityRegistry.from([orm_integration_1.UserCredentials, orm_integration_1.User]));
    (0, globals_1.expect)(userCredentials.getColumn('user').isPrimaryKey).toBe(true);
    (0, globals_1.expect)(userCredentials.getColumn('user').type).toBe('integer');
});
(0, globals_1.test)('class basic', async () => {
    class Product {
        constructor(name) {
            this.name = name;
            this.id = 0;
            this.created = new Date;
        }
    }
    Product.__type = [() => __ΩPrimaryKey, 'id', function () { return 0; }, 'created', function () { return new Date; }, 'name', 'constructor', 'P\'n!K3">#T3$>%P&2&:"0\'5'];
    const database = await (0, factory_1.databaseFactory)([Product]);
    const product1 = (0, type_1.cast)({ id: 1, name: 'Yes', created: new Date() }, undefined, undefined, undefined, [() => Product, 'P7!']);
    const product2 = (0, type_1.cast)({ id: 2, name: 'Wow', created: new Date() }, undefined, undefined, undefined, [() => Product, 'P7!']);
    const product3 = (0, type_1.cast)({ id: 3, name: 'asdadasd', created: new Date() }, undefined, undefined, undefined, [() => Product, 'P7!']);
    {
        const session = database.createSession();
        (0, globals_1.expect)(await session.query(Product).count()).toBe(0);
        session.add(product1, product2, product3);
        await session.commit();
        (0, globals_1.expect)(await session.query(Product).count()).toBe(3);
        product1.name = 'Changed';
        await session.commit();
        (0, globals_1.expect)(await session.query(Product).count()).toBe(3);
        (0, globals_1.expect)((await session.query(Product).disableIdentityMap().filter(product1).findOne()).name).toBe('Changed');
    }
    {
        const session = database.createSession();
        const user1db = await session.query(Product).filter({ id: product1.id }).findOne();
        (0, globals_1.expect)(user1db.name).toBe('Changed');
    }
    {
        const session = database.createSession();
        (0, globals_1.expect)((await session.query(Product).deleteMany()).modified).toBe(3);
        (0, globals_1.expect)((await session.query(Product).deleteMany()).modified).toBe(0);
    }
});
(0, globals_1.test)('interface basic', async () => {
    const __ΩProduct = [() => __ΩEntity, "product", 'name', () => __ΩPrimaryKey, 'id', 'created', 'PP."4#Mo!"P\'n$K4%&4#T4&M'];
    const database = await (0, factory_1.databaseFactory)([(0, type_1.typeOf)([], [() => __ΩProduct, 'n!'])]);
    const product1 = { id: 1, name: 'Yes', created: new Date() };
    const product2 = { id: 2, name: 'Wow', created: new Date() };
    const product3 = { id: 3, name: 'asdadasd', created: new Date() };
    {
        const session = database.createSession();
        (0, globals_1.expect)(await session.query([() => __ΩProduct, 'n!']).count()).toBe(0);
        session.add(product1, product2, product3);
        await session.commit();
        (0, globals_1.expect)(await session.query([() => __ΩProduct, 'n!']).count()).toBe(3);
        product1.name = 'Changed';
        await session.commit();
        (0, globals_1.expect)(await session.query([() => __ΩProduct, 'n!']).count()).toBe(3);
        (0, globals_1.expect)((await session.query([() => __ΩProduct, 'n!']).disableIdentityMap().filter(product1).findOne()).name).toBe('Changed');
    }
    {
        const session = database.createSession();
        const user1db = await session.query([() => __ΩProduct, 'n!']).filter({ id: product1.id }).findOne();
        (0, globals_1.expect)(user1db.name).toBe('Changed');
    }
    {
        const session = database.createSession();
        (0, globals_1.expect)((await session.query([() => __ΩProduct, 'n!']).deleteMany()).modified).toBe(3);
        (0, globals_1.expect)((await session.query([() => __ΩProduct, 'n!']).deleteMany()).modified).toBe(0);
    }
});
(0, globals_1.test)('sqlite autoincrement', async () => {
    let User = class User {
        constructor(name) {
            this.name = name;
            this.created = new Date;
        }
    };
    User.__type = [() => __ΩPrimaryKey, () => __ΩAutoIncrement, 'id', 'created', function () { return new Date; }, 'name', 'constructor', 'P\'n!n"K3#8T3$>%P&2&:"0\'5'];
    User = __decorate([
        type_1.entity.name('sqlite-user'),
        __metadata("design:paramtypes", [String])
    ], User);
    const database = await (0, factory_1.databaseFactory)([User]);
    const session = database.createSession();
    (0, globals_1.expect)(await session.query(User).count()).toBe(0);
    const peter = new User('Peter');
    const herbert = new User('Herbert');
    session.add(peter);
    session.add(herbert);
    await session.commit();
    (0, globals_1.expect)(await session.query(User).count()).toBe(2);
    (0, globals_1.expect)(peter.id).toBe(1);
    (0, globals_1.expect)(herbert.id).toBe(2);
    (0, globals_1.expect)(await session.query(User).count()).toBe(2);
});
(0, globals_1.test)('sqlite relation', async () => {
    let Author = class Author {
        constructor(id, name) {
            this.id = id;
            this.name = name;
            this.created = new Date;
        }
    };
    Author.__type = ['created', function () { return new Date; }, () => __ΩPrimaryKey, 'id', 'name', 'constructor', 'T3!>"PP\'n#K2$:&2%:"0&5'];
    Author = __decorate([
        type_1.entity.name('sqlite-author'),
        __metadata("design:paramtypes", [Object, String])
    ], Author);
    let Book = class Book {
        constructor(id, author, name) {
            this.id = id;
            this.author = author;
            this.name = name;
        }
    };
    Book.__type = [() => __ΩPrimaryKey, 'id', () => Author, () => __ΩReference, 'author', 'name', 'constructor', 'PP\'n!K2":PP7#n$K2%:&2&:"0\'5'];
    Book = __decorate([
        type_1.entity.name('sqlite-book'),
        __metadata("design:paramtypes", [Object, Object, String])
    ], Book);
    const database = await (0, factory_1.databaseFactory)([Author, Book]);
    const session = database.createSession();
    (0, globals_1.expect)(await session.query(Author).count()).toBe(0);
    const peter = new Author(1, 'Peter');
    const herbert = new Author(2, 'Herbert');
    session.add(peter);
    session.add(herbert);
    const book1 = new Book(1, peter, 'Peters book');
    session.add(book1);
    await session.commit();
    (0, globals_1.expect)(await session.query(Author).count()).toBe(2);
    (0, globals_1.expect)(await session.query(Book).count()).toBe(1);
});
(0, globals_1.test)('transaction', async () => {
    const sqlite = new sqlite_adapter_1.SQLiteDatabaseAdapter(':memory:');
    {
        const t1 = new sqlite_adapter_1.SQLiteDatabaseTransaction();
        const c1 = await sqlite.connectionPool.getConnection(undefined, t1);
        const c2 = await sqlite.connectionPool.getConnection();
        (0, globals_1.expect)(c1 === c2).toBe(false);
        c1.release(); //this doesnt release anything
        c2.release();
        (0, globals_1.expect)(c1.released).toBe(false);
        {
            const c3 = await sqlite.connectionPool.getConnection();
            (0, globals_1.expect)(c3 === c1).toBe(false);
        }
        c1.transaction.ended = true;
        c1.release(); //now it can be released
        {
            const c3 = await sqlite.connectionPool.getConnection();
            (0, globals_1.expect)(c3 === c1).toBe(true);
        }
    }
});
(0, globals_1.test)('connection pool', async () => {
    const sqlite = new sqlite_adapter_1.SQLiteDatabaseAdapter(':memory:');
    {
        const c1 = await sqlite.connectionPool.getConnection();
        const c2 = await sqlite.connectionPool.getConnection();
        (0, globals_1.expect)(c1 === c2).toBe(false);
        c1.release();
        c2.release();
        const c3 = await sqlite.connectionPool.getConnection();
        (0, globals_1.expect)(c3 === c1).toBe(true);
    }
    {
        const c1 = await sqlite.connectionPool.getConnection();
        const c2 = await sqlite.connectionPool.getConnection();
        const c3 = await sqlite.connectionPool.getConnection();
        const c4 = await sqlite.connectionPool.getConnection();
        const c5 = await sqlite.connectionPool.getConnection();
        const c6 = await sqlite.connectionPool.getConnection();
        const c7 = await sqlite.connectionPool.getConnection();
        const c8 = await sqlite.connectionPool.getConnection();
        const c9 = await sqlite.connectionPool.getConnection();
        const c10 = await sqlite.connectionPool.getConnection();
        // this blocks
        let c11;
        sqlite.connectionPool.getConnection().then(Object.assign((c) => {
            c11 = c;
        }, { __type: ['c', '', 'P"2!"/"'] }));
        let c12;
        sqlite.connectionPool.getConnection().then(Object.assign((c) => {
            c12 = c;
        }, { __type: ['c', '', 'P"2!"/"'] }));
        await (0, core_1.sleep)(0.01);
        (0, globals_1.expect)(c11).toBe(undefined);
        (0, globals_1.expect)(c12).toBe(undefined);
        c1.release();
        await (0, core_1.sleep)(0.01);
        (0, globals_1.expect)(c11).toBe(c1);
        c2.release();
        await (0, core_1.sleep)(0.01);
        (0, globals_1.expect)(c12).toBe(c2);
    }
});
//# sourceMappingURL=sqlite.spec.js.map