import { AbstractClassType, ClassType } from '@deepkit/core';
import { DatabaseAdapter, DatabaseLogger, DatabasePersistenceChangeSet, DatabaseSession, DatabaseTransaction, DeleteResult, OrmEntity, PatchResult } from '@deepkit/orm';
import { DefaultPlatform, SQLConnection, SQLConnectionPool, SQLDatabaseAdapter, SQLDatabaseQuery, SQLDatabaseQueryFactory, SQLPersistence, SQLQueryModel, SQLQueryResolver, SQLStatement } from '@deepkit/sql';
import { Changes, ReceiveType, ReflectionClass } from '@deepkit/type';
import sqlite3 from 'better-sqlite3';
import { SQLitePlatform } from './sqlite-platform';
import { Stopwatch } from '@deepkit/stopwatch';
export declare class SQLiteStatement extends SQLStatement {
    protected logger: DatabaseLogger;
    protected sql: string;
    protected stmt: sqlite3.Statement;
    protected stopwatch?: Stopwatch | undefined;
    constructor(logger: DatabaseLogger, sql: string, stmt: sqlite3.Statement, stopwatch?: Stopwatch | undefined);
    get(params?: any[]): Promise<any>;
    all(params?: any[]): Promise<any[]>;
    release(): void;
}
export declare class SQLiteDatabaseTransaction extends DatabaseTransaction {
    connection?: SQLiteConnection;
    begin(): Promise<void>;
    commit(): Promise<void>;
    rollback(): Promise<void>;
}
export declare class SQLiteConnection extends SQLConnection {
    protected dbPath: string;
    platform: SQLitePlatform;
    protected changes: number;
    db: sqlite3.Database;
    static DatabaseConstructor: any;
    constructor(connectionPool: SQLConnectionPool, dbPath: string, logger?: DatabaseLogger, transaction?: DatabaseTransaction, stopwatch?: Stopwatch);
    prepare(sql: string): Promise<SQLiteStatement>;
    protected handleError(error: Error | string): void;
    run(sql: string, params?: any[]): Promise<void>;
    exec(sql: string): Promise<void>;
    getChanges(): Promise<number>;
}
export declare class SQLiteConnectionPool extends SQLConnectionPool {
    protected dbPath: string;
    maxConnections: number;
    protected queue: ((connection: SQLiteConnection) => void)[];
    protected firstConnection?: SQLiteConnection;
    constructor(dbPath: string);
    close(): void;
    protected createConnection(logger?: DatabaseLogger, transaction?: SQLiteDatabaseTransaction, stopwatch?: Stopwatch): SQLiteConnection;
    getConnection(logger?: DatabaseLogger, transaction?: SQLiteDatabaseTransaction, stopwatch?: Stopwatch): Promise<SQLiteConnection>;
    release(connection: SQLiteConnection): void;
}
export declare class SQLitePersistence extends SQLPersistence {
    protected platform: DefaultPlatform;
    connectionPool: SQLiteConnectionPool;
    constructor(platform: DefaultPlatform, connectionPool: SQLiteConnectionPool, database: DatabaseSession<any>);
    protected getInsertSQL(classSchema: ReflectionClass<any>, fields: string[], values: string[]): string;
    batchUpdate<T extends OrmEntity>(classSchema: ReflectionClass<T>, changeSets: DatabasePersistenceChangeSet<T>[]): Promise<void>;
    protected populateAutoIncrementFields<T>(classSchema: ReflectionClass<T>, items: T[]): Promise<void>;
}
export declare class SQLiteQueryResolver<T extends OrmEntity> extends SQLQueryResolver<T> {
    protected connectionPool: SQLiteConnectionPool;
    protected platform: DefaultPlatform;
    constructor(connectionPool: SQLiteConnectionPool, platform: DefaultPlatform, classSchema: ReflectionClass<T>, session: DatabaseSession<DatabaseAdapter>);
    delete(model: SQLQueryModel<T>, deleteResult: DeleteResult<T>): Promise<void>;
    patch(model: SQLQueryModel<T>, changes: Changes<T>, patchResult: PatchResult<T>): Promise<void>;
}
export declare class SQLiteDatabaseQuery<T> extends SQLDatabaseQuery<T> {
}
export declare class SQLiteDatabaseQueryFactory extends SQLDatabaseQueryFactory {
    protected connectionPool: SQLiteConnectionPool;
    constructor(connectionPool: SQLiteConnectionPool, platform: DefaultPlatform, databaseSession: DatabaseSession<any>);
    createQuery<T extends OrmEntity>(type?: ReceiveType<T> | ClassType<T> | AbstractClassType<T> | ReflectionClass<T>): SQLiteDatabaseQuery<T>;
}
export declare class SQLiteDatabaseAdapter extends SQLDatabaseAdapter {
    protected sqlitePath: string;
    readonly connectionPool: SQLiteConnectionPool;
    readonly platform: SQLitePlatform;
    constructor(sqlitePath?: string);
    getInsertBatchSize(schema: ReflectionClass<any>): Promise<number>;
    getName(): string;
    getSchemaName(): string;
    createTransaction(session: DatabaseSession<this>): SQLiteDatabaseTransaction;
    createPersistence(session: DatabaseSession<any>): SQLPersistence;
    queryFactory(databaseSession: DatabaseSession<any>): SQLiteDatabaseQueryFactory;
    disconnect(force?: boolean): void;
}
