var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { __ΩPrimaryKey as __ΩPrimaryKey } from '@deepkit/type';
import { __ΩAutoIncrement as __ΩAutoIncrement } from '@deepkit/type';
import { __ΩReference as __ΩReference } from '@deepkit/type';
import { __ΩEntity as __ΩEntity } from '@deepkit/type';
import { expect, test } from '@jest/globals';
import { SQLitePlatform } from '../src/sqlite-platform';
import { databaseFactory } from './factory';
import { User, UserCredentials } from '@deepkit/orm-integration';
import { SQLiteDatabaseAdapter, SQLiteDatabaseTransaction } from '../src/sqlite-adapter';
import { sleep } from '@deepkit/core';
import { cast, entity, ReflectionClass, typeOf } from '@deepkit/type';
import { DatabaseEntityRegistry } from '@deepkit/orm';
test('reflection circular reference', () => {
    const user = ReflectionClass.from(User);
    const credentials = ReflectionClass.from(UserCredentials);
    expect(user.getProperty('credentials').isBackReference()).toBe(true);
    expect(user.getProperty('credentials').getResolvedReflectionClass()).toBe(credentials);
    const userProperty = credentials.getProperty('user');
    expect(userProperty.getResolvedReflectionClass()).toBe(user);
    expect(userProperty.isPrimaryKey()).toBe(true);
    expect(userProperty.isReference()).toBe(true);
});
test('tables', () => {
    const [user] = new SQLitePlatform().createTables(DatabaseEntityRegistry.from([User]));
    expect(user.getColumn('birthdate').isNotNull).toBe(false);
    const [userCredentials] = new SQLitePlatform().createTables(DatabaseEntityRegistry.from([UserCredentials, User]));
    expect(userCredentials.getColumn('user').isPrimaryKey).toBe(true);
    expect(userCredentials.getColumn('user').type).toBe('integer');
});
test('class basic', async () => {
    class Product {
        constructor(name) {
            this.name = name;
            this.id = 0;
            this.created = new Date;
        }
    }
    Product.__type = [() => __ΩPrimaryKey, 'id', function () { return 0; }, 'created', function () { return new Date; }, 'name', 'constructor', 'P\'n!K3">#T3$>%P&2&:"0\'5'];
    const database = await databaseFactory([Product]);
    const product1 = cast({ id: 1, name: 'Yes', created: new Date() }, undefined, undefined, undefined, [() => Product, 'P7!']);
    const product2 = cast({ id: 2, name: 'Wow', created: new Date() }, undefined, undefined, undefined, [() => Product, 'P7!']);
    const product3 = cast({ id: 3, name: 'asdadasd', created: new Date() }, undefined, undefined, undefined, [() => Product, 'P7!']);
    {
        const session = database.createSession();
        expect(await session.query(Product).count()).toBe(0);
        session.add(product1, product2, product3);
        await session.commit();
        expect(await session.query(Product).count()).toBe(3);
        product1.name = 'Changed';
        await session.commit();
        expect(await session.query(Product).count()).toBe(3);
        expect((await session.query(Product).disableIdentityMap().filter(product1).findOne()).name).toBe('Changed');
    }
    {
        const session = database.createSession();
        const user1db = await session.query(Product).filter({ id: product1.id }).findOne();
        expect(user1db.name).toBe('Changed');
    }
    {
        const session = database.createSession();
        expect((await session.query(Product).deleteMany()).modified).toBe(3);
        expect((await session.query(Product).deleteMany()).modified).toBe(0);
    }
});
test('interface basic', async () => {
    const __ΩProduct = [() => __ΩEntity, "product", 'name', () => __ΩPrimaryKey, 'id', 'created', 'PP."4#Mo!"P\'n$K4%&4#T4&M'];
    const database = await databaseFactory([typeOf([], [() => __ΩProduct, 'n!'])]);
    const product1 = { id: 1, name: 'Yes', created: new Date() };
    const product2 = { id: 2, name: 'Wow', created: new Date() };
    const product3 = { id: 3, name: 'asdadasd', created: new Date() };
    {
        const session = database.createSession();
        expect(await session.query([() => __ΩProduct, 'n!']).count()).toBe(0);
        session.add(product1, product2, product3);
        await session.commit();
        expect(await session.query([() => __ΩProduct, 'n!']).count()).toBe(3);
        product1.name = 'Changed';
        await session.commit();
        expect(await session.query([() => __ΩProduct, 'n!']).count()).toBe(3);
        expect((await session.query([() => __ΩProduct, 'n!']).disableIdentityMap().filter(product1).findOne()).name).toBe('Changed');
    }
    {
        const session = database.createSession();
        const user1db = await session.query([() => __ΩProduct, 'n!']).filter({ id: product1.id }).findOne();
        expect(user1db.name).toBe('Changed');
    }
    {
        const session = database.createSession();
        expect((await session.query([() => __ΩProduct, 'n!']).deleteMany()).modified).toBe(3);
        expect((await session.query([() => __ΩProduct, 'n!']).deleteMany()).modified).toBe(0);
    }
});
test('sqlite autoincrement', async () => {
    let User = class User {
        constructor(name) {
            this.name = name;
            this.created = new Date;
        }
    };
    User.__type = [() => __ΩPrimaryKey, () => __ΩAutoIncrement, 'id', 'created', function () { return new Date; }, 'name', 'constructor', 'P\'n!n"K3#8T3$>%P&2&:"0\'5'];
    User = __decorate([
        entity.name('sqlite-user'),
        __metadata("design:paramtypes", [String])
    ], User);
    const database = await databaseFactory([User]);
    const session = database.createSession();
    expect(await session.query(User).count()).toBe(0);
    const peter = new User('Peter');
    const herbert = new User('Herbert');
    session.add(peter);
    session.add(herbert);
    await session.commit();
    expect(await session.query(User).count()).toBe(2);
    expect(peter.id).toBe(1);
    expect(herbert.id).toBe(2);
    expect(await session.query(User).count()).toBe(2);
});
test('sqlite relation', async () => {
    let Author = class Author {
        constructor(id, name) {
            this.id = id;
            this.name = name;
            this.created = new Date;
        }
    };
    Author.__type = ['created', function () { return new Date; }, () => __ΩPrimaryKey, 'id', 'name', 'constructor', 'T3!>"PP\'n#K2$:&2%:"0&5'];
    Author = __decorate([
        entity.name('sqlite-author'),
        __metadata("design:paramtypes", [Object, String])
    ], Author);
    let Book = class Book {
        constructor(id, author, name) {
            this.id = id;
            this.author = author;
            this.name = name;
        }
    };
    Book.__type = [() => __ΩPrimaryKey, 'id', () => Author, () => __ΩReference, 'author', 'name', 'constructor', 'PP\'n!K2":PP7#n$K2%:&2&:"0\'5'];
    Book = __decorate([
        entity.name('sqlite-book'),
        __metadata("design:paramtypes", [Object, Object, String])
    ], Book);
    const database = await databaseFactory([Author, Book]);
    const session = database.createSession();
    expect(await session.query(Author).count()).toBe(0);
    const peter = new Author(1, 'Peter');
    const herbert = new Author(2, 'Herbert');
    session.add(peter);
    session.add(herbert);
    const book1 = new Book(1, peter, 'Peters book');
    session.add(book1);
    await session.commit();
    expect(await session.query(Author).count()).toBe(2);
    expect(await session.query(Book).count()).toBe(1);
});
test('transaction', async () => {
    const sqlite = new SQLiteDatabaseAdapter(':memory:');
    {
        const t1 = new SQLiteDatabaseTransaction();
        const c1 = await sqlite.connectionPool.getConnection(undefined, t1);
        const c2 = await sqlite.connectionPool.getConnection();
        expect(c1 === c2).toBe(false);
        c1.release(); //this doesnt release anything
        c2.release();
        expect(c1.released).toBe(false);
        {
            const c3 = await sqlite.connectionPool.getConnection();
            expect(c3 === c1).toBe(false);
        }
        c1.transaction.ended = true;
        c1.release(); //now it can be released
        {
            const c3 = await sqlite.connectionPool.getConnection();
            expect(c3 === c1).toBe(true);
        }
    }
});
test('connection pool', async () => {
    const sqlite = new SQLiteDatabaseAdapter(':memory:');
    {
        const c1 = await sqlite.connectionPool.getConnection();
        const c2 = await sqlite.connectionPool.getConnection();
        expect(c1 === c2).toBe(false);
        c1.release();
        c2.release();
        const c3 = await sqlite.connectionPool.getConnection();
        expect(c3 === c1).toBe(true);
    }
    {
        const c1 = await sqlite.connectionPool.getConnection();
        const c2 = await sqlite.connectionPool.getConnection();
        const c3 = await sqlite.connectionPool.getConnection();
        const c4 = await sqlite.connectionPool.getConnection();
        const c5 = await sqlite.connectionPool.getConnection();
        const c6 = await sqlite.connectionPool.getConnection();
        const c7 = await sqlite.connectionPool.getConnection();
        const c8 = await sqlite.connectionPool.getConnection();
        const c9 = await sqlite.connectionPool.getConnection();
        const c10 = await sqlite.connectionPool.getConnection();
        // this blocks
        let c11;
        sqlite.connectionPool.getConnection().then(Object.assign((c) => {
            c11 = c;
        }, { __type: ['c', '', 'P"2!"/"'] }));
        let c12;
        sqlite.connectionPool.getConnection().then(Object.assign((c) => {
            c12 = c;
        }, { __type: ['c', '', 'P"2!"/"'] }));
        await sleep(0.01);
        expect(c11).toBe(undefined);
        expect(c12).toBe(undefined);
        c1.release();
        await sleep(0.01);
        expect(c11).toBe(c1);
        c2.release();
        await sleep(0.01);
        expect(c12).toBe(c2);
    }
});
//# sourceMappingURL=sqlite.spec.js.map